/*
 ͸
  Source file: NSXTOOLS.PRG                                                
  Description: Some handy UDF's for use with the HiPer-SIx RDD             
  Last Update: 12/01/93                                                    
  Notice     : Copyright 1993 - SuccessWare 90, Inc.                       
                                                                           
  Compile    : CLIPPER nsxtools /n/w                                       
                                                                           
 Ĵ
                                                                           
  Odometer()     - For use with the INDEX ON command's OPTION clause.      
                                                                           
  Dots()         - Also for use with the INDEX ON command's OPTION clause. 
                                                                           
  IndexBar()     - Another progress meter like Odometer() for indexing     
                                                                           
  ShowIt()       - Yet another                                             
                                                                           
 ;
   
*/

#include "inkey.ch"
#include "dbfnsx.ch"

#define TRUE  .T.
#define FALSE .F.

/*==========================================================================
  Odometer():

  By Jeff Jochum and Loren Scott

  This is a sample function designed to be used as an "odometer" with the
  OPTION clause when creating or re-creating indexes with the INDEX ON or
  REINDEX commands.  It is "intelligent" in that it knows whether there is
  a Conditional Index or Scoped Index active and uses the appropriate
  ScopeInfo() values rather than the Sx_KeyCount() function.

  Note: This UDF depends on data from the ScopeInfo() function defined
        later in this program.
---------------------------------------------------------------------------*/

FUNC Odometer( nRecCount, nLineNum, nColNum )

LOCAL nPctDone
STATIC nLine := 0
STATIC nRecs := 0
STATIC nRec := 0
STATIC nCol := 0
STATIC cScreen

// Make sure DBFNSX driver is being used; This only works with DBFNSX
IF IndexExt() != ".NSX"
  Return(TRUE)
ENDIF

// Check for <Escape> key pressed
IF Inkey() == K_ESC
  IF Alert("Abort Process?", {"Yes","No"}) == 1
    RETURN (FALSE)
  ENDIF
ENDIF

// BOF is TRUE on initial OPTION UDF call.  Do setup here.
IF BOF()
   nLine := IF(nLineNum == NIL, MaxRow()/2, Min( nLineNum, MaxRow() ))
   nCol  := IF(nColNum  == NIL, 15, Min( nColNum, 15 ))

   // If TagName != Indexname (it's a tag) and reindexing...
   IF (!(Sx_I_TagName() == Sx_I_IndexName()) .AND. Sx_IsReindex() )
      nRecs := IF(nRecCount == NIL, RecCount(), nRecCount)
   ELSE
      nRecs := IF(nRecCount == NIL, Sx_KeyCount(), nRecCount )
   ENDIF
   nRec := 0
   cScreen := SaveScreen( nLine-2, nCol-1, nLine+2, nCol+52 )


   Scroll( nLine-2, nCol-1, nLine+2, nCol+52 )  // Clear the display area
   @ nLine-2, nCol-1 TO nLine+2, nCol+52        // Draw a box for the odometer
   @ nLine-2, nCol+1 SAY Sx_I_IndexName()       // Show index name
   @ nLine-2, nCol+50-Len( Sx_I_TagName()) SAY Sx_I_TagName()  // Tag Name

// If EOF is TRUE then the indexing has finished.  Clean up.
ELSEIF EOF()
   @ nLine-1, nCol+1 say Replicate(" ",47) + "100%"
   @ nLine, nCol+1 say Replicate("", 50)
   @ nLine+1,nCol say "Processed Keys " +;
                     Left(LTrim(Str(nRecs))+Space(10),10)
   @ nLine+1,nCol+26 say " Keys Included " +;
                     Left( LTrim( Str( Sx_KeysIncluded())) + Space(10),10)
   // Wait a second so they can see that the process is finished
   Inkey(1)
   RestScreen( nLine-2, nCol-1, nLine+2, nCol+52, cScreen )

// In progress... show current status.
ELSE
   // Increase record # by step count
   nRec += Sx_Step()

   // Calculate percentage done
   nPctDone := (nRec / nRecs) * 100
   @ nLine-1,nCol+1 say ;
     Replicate(" ", ((nPctDone-.5) / 2) -2) + Str(nPctDone,3)+"%"
   @ nLine,nCol+1 say ;
     Replicate("", ((nPctDone-.5) / 2 )) + "" +;
     Replicate("", ( 50 - (nPctDone / 2)))
   @ nLine+1,nCol say ;
     "Processing Key "+Left(LTrim(Str(MIN(nRec,nRecs))+Space(10)),10);
     +" of "+Left(LTrim(Str(nRecs))+Space(10),10) + Space(6)
ENDIF

Return(TRUE)


/*==========================================================================
  Dots():

  By Loren Scott

  This is a sample function designed to be used with the OPTION clause when
  creating or re-creating indexes with the INDEX ON or REINDEX commands.  It
  simply displays a string of dots starting from the specified starting
  column (nStartCol) to the right edge of the screen, and then clears the
  dots from the right side back to the starting column again.

  This function does provide any specific information on what file or tag
  is being created, but it _does_ show that SOMETHING is being done.  And...
  it looks kinda slick!

  EXAMPLE:    USE test VIA "DBFNSX"
              ? "Creating TEST.NSX:"
              nCol := Col()
              INDEX ON last+first TO TEST OPTION Dots( nCol ) STEP 10

--------------------------------------------------------------------------*/

FUNC Dots( nStartCol )

LOCAL nCol := Col()
STATIC lOut := TRUE

IF nCol == nStartCol
   lOut := TRUE
ELSEIF nCol == MaxCol()-2
   DevPos( Row(), nCol-1 )
   QQOut(" ")
   lOut := FALSE
ENDIF

IF lOut
   QQOut(".")
ELSE
   DevPos( Row(), nCol-2 )
   QQOut(" ")
ENDIF

Return(TRUE)


/*===================================================================//

  IndexBar():

    This function was provided to SuccessWare for inclusion with the
    DBFNSX library by it's author:

       Dr Ben Williams  -  Perth, Western Australia

    Based on ideas gathered from the Australian Clipper community.
    Ben can be contacted on CIS c/o STR Computers 100033,1424

----------------------------------------------------------------------*/

#define BOXTOP   8
#define BOXLEFT 10

FUNC IndexBar()

STATIC aScrn, cSaveColor, nLast

LOCAL nRecs := Sx_KeysIncluded() , cStd := "GR+/B"

// BOF() is TRUE on first call to OPTION UDF.  Use this for setup.
IF Bof()
  nLast := LastRec()
  cSaveColor := SetColor( "GR+/B,R+/W" )
  aScrn := { BOXTOP, BOXLEFT, BOXTOP + 11, BOXLEFT + 60 ,;
            SaveScreen( BOXTOP, BOXLEFT, BOXTOP + 11, BOXLEFT + 60 ) }
  DispBox( BOXTOP, BOXLEFT, BOXTOP + 11, BOXLEFT + 60 , 2 )
  Scroll( BOXTOP + 1, BOXLEFT + 1, BOXTOP + 10, BOXLEFT + 59 )
  SetPos( BOXTOP + 2, BOXLEFT + 5)
  DevOut( "Creating Index      :               ")
  SetPos( BOXTOP + 3, BOXLEFT + 5)
  DevOut( "Tag name            :               ")
  SetPos( BOXTOP + 4, BOXLEFT + 5)
  DevOut( "Total Records       :               ")
  SetPos( BOXTOP + 5, BOXLEFT + 5)
  DevOut( "Current Record      :               ")
  SetPos( BOXTOP + 6, BOXLEFT + 5)
  DevOut( "Percentage complete :               ")
  SetPos( BOXTOP + 8, BOXLEFT + 5)
  DevOut( "0%  50%  100%")

  // Clear the area, create the guage
  Scroll(BOXTOP + 2, BOXLEFT  + 27, BOXTOP + 11 - 5, BOXLEFT + 60 - 1, 0 )
  SetPos( BOXTOP + 2, BOXLEFT  + 28)
  DevOut( Sx_I_IndexName())
  SetPos( BOXTOP + 3, BOXLEFT  + 28)
  DevOut( Sx_I_TagName()  )
  SetPos( BOXTOP + 4, BOXLEFT  + 28)
  DevOut( LTrim( Transform( nLast,"999,999,999")) )
  aReplicate( BOXTOP + 9, BOXLEFT + 5, chr(219), 50, "N/N" )

ELSEIF Eof()
  SetColor( cSaveColor )
  RestScreen( aScrn[1], aScrn[2], aScrn[3], aScrn[4], aScrn[5] )

ELSE
  // Show how many keys have been included so far
  SetPos( BOXTOP + 5, BOXLEFT + 28)
  DevOut( LTrim( Transform( nRecs ,  "999,999,999")), cStd )
  SetPos( BOXTOP + 6, BOXLEFT + 28)
  DevOut( LTrim( Transform( ( nRecs / nLast) * 100 ,  "999")),cStd )

  // Show how far we have got
  aReplicate( BOXTOP + 9, BOXLEFT + 5, chr(219),;
              int(( nRecs / nLast) * 50 ), "GR+/W" )
ENDIF

Return(TRUE)


FUNC aReplicate( nR, nC, cSt, nLen, cColor )

SetPos( nR, nC )
DevOut( Replicate( cSt, nLen ), cColor )

Return(NIL)

/*==================================================================//

  ShowIt():

  By Loren Scott

  Yet another progress display UDF for use with the INDEX command's
  OPTION clause.  This one does a neat spinning bar effect.

-------------------------------------------------------------------*/

#include "BOX.CH"

FUNCTION ShowIt()

LOCAL aBogus := { "-", "\", "|", "/" }
STATIC nCount, cColor, lBlink, cScreen, nRow, nCol

IF Bof()
  cScreen := SaveScreen( 0,0,MaxRow(),MaxCol() )
  lBlink := SetBlink(FALSE)
  cColor := SetColor("b/w*")
  @11,20,13,59 BOX B_SINGLE+" "
  @11,35 SAY " Working   "
  nCount := 0
  nRow := Row()
  nCol := Col()
ELSEIF Eof()
  SetColor( cColor )
  SetBlink( lBlink )
  RestScreen( 0,0,MaxRow(),MaxCol(),cScreen )
  DevPos( nRow, nCol )
ELSE
  nCount++
  @ 11,44 SAY aBogus[ nCount ]
  @ 12,25 SAY Sx_I_IndexName() + " : " + Sx_I_TagName()
  nCount := iif( nCount == 4, 0, nCount )
ENDIF

Return(TRUE)

//------------------------------------------------------------------//

